@@ -7,7 +7,7 @@ from textwrap import dedent
 
 from gtunrealdevice.utils import File
 
-__version__ = '0.1.0'
+__version__ = '0.2.1'
 version = __version__
 __edition__ = 'Community'
 edition = __edition__
@@ -28,6 +28,43 @@ class Data:
     devices_info_filename = File.get_path(app_directory, 'devices_info.yaml')
     serialized_filename = File.get_path(app_directory, 'serialized_data.yaml')
 
+    # app sample data
+    sample_devices_info_text = dedent("""
+        ####################################################################
+        # sample devices info                                              #
+        # Note: name, login, testcases, and configs nodes are optional     #
+        ####################################################################
+        host_address_1:
+          name: host_name (optional)
+          login: |-
+            output_of_login (optional)
+          cmdlines:
+            cmdline_1: |-
+              line 1 output_of_cmdline_1
+              ...
+              line n output_of_cmdline_1
+            cmdline_k_for_multiple_output:
+              - |-
+                line 1 - output_of_cmdline_k
+                ...
+                line n - output_of_cmdline_k
+              - |-
+                line 1 - other_output_of_cmdline_k
+                ...
+                line n - other_output_of_cmdline_k
+          testcases:
+            name_of_testcase_1:
+              cmdline_1: |-
+                line 1 output_of_cmdline_1_of_testcase_1
+                ...
+                line n output_of_cmdline_1_of_testcase_1
+          configs:
+            cfg_1_reference: |-
+              line 1 of cfg_1 
+              ...
+              line n of cfg_1
+    """).strip()
+
     # main app
     main_app_text = 'gtunrealdevice v{}'.format(version)
 
