@@ -5,7 +5,6 @@ import yaml
 import functools
 from os import path
 from datetime import datetime
-from textwrap import dedent
 
 from gtunrealdevice.config import Data
 from gtunrealdevice.exceptions import WrapperError
@@ -14,6 +13,7 @@ from gtunrealdevice.exceptions import UnrealDeviceConnectionError
 from gtunrealdevice.exceptions import UnrealDeviceOfflineError
 
 from gtunrealdevice.utils import Printer
+from gtunrealdevice.utils import Misc
 
 
 def check_active_device(func):
@@ -41,6 +41,7 @@ def check_active_device(func):
                     result = func(*args, **kwargs)
                     return result
                 else:
+                    device.success_code = 1
                     fmt = '{} device is offline.'
                     raise UnrealDeviceOfflineError(fmt.format(device.name))
             else:
@@ -120,6 +121,53 @@ class DevicesData(dict):
         with open(path.expanduser(filename), 'w') as stream:
             self and yaml.safe_dump(dict(self), stream)
 
+    def get_address_from_name(self, name):
+        """Get device address from device name
+
+        Parameters
+        ----------
+        name (str): a device name
+
+        Returns
+        -------
+        str: device address or original name
+        """
+        name = str(name).strip()
+
+        if not name and len(self) == 1:
+            return list(self)[0]
+
+        if name in self:
+            return name
+        else:
+            for addr, node in self.items():
+                if node.get('name') == name:
+                    return addr
+            return name
+
+    def is_testcase_exist(self, address, testcase):
+        """Check if test case is existed.
+
+        Parameters
+        ----------
+        address (str):
+        testcase (str): a test case name
+
+        Returns
+        -------
+        bool: True if found, otherwise False
+        """
+        testcase = str(testcase).strip()
+        if not testcase:
+            return False
+
+        address = self.get_address_from_name(address)
+        if address not in self:
+            return False
+
+        testcases = self.get(address).get('testcases', dict())
+        return testcase in testcases
+
     def is_valid_file(self, filename):
         """Check filename
 
@@ -193,41 +241,7 @@ class DevicesData(dict):
         return True
 
     def get_sample_device_info_format(self):    # noqa
-        text = dedent('''
-            ####################################################################
-            # sample device info                                               #
-            # name, login, and testcases nodes are optional                    #
-            ####################################################################
-            host_address_1:
-              name: host_name (optional)
-              login: |-
-                output_of_login (optional)
-              cmdlines:
-                cmdline_1: |-
-                  line 1 output_of_cmdline_1
-                  ...
-                  line n output_of_cmdline_1
-                cmdline_k_for_multiple_output:
-                  - |-
-                    line 1 - output_of_cmdline_k
-                    ...
-                    line n - output_of_cmdline_k
-                  - |-
-                    line 1 - other_output_of_cmdline_k
-                    ...
-                    line n - other_output_of_cmdline_k
-              testcases:
-                name_of_testcase_1:
-                  cmdline_1: |-
-                    line 1 output_of_cmdline_1_of_testcase_1
-                    ...
-                    line n output_of_cmdline_1_of_testcase_1
-              configs:
-                cfg_1: |-
-                  line 1 of cfg_1 
-                  ...
-                  line n of cfg_1
-        ''').strip()
+        text = Data.sample_devices_info_text
         return text
 
     def get_data(self, data):       # noqa
@@ -286,7 +300,7 @@ class DevicesData(dict):
                 self[device] = dict(cmdlines={cmdline: output})
 
     def view(self, device='', cmdlines=False, testcase='', testcases=False):
-        lst = ['Devices Info:']
+        lst = ['Devices Data:']
         lst.extend(['  - Location: {}'.format(fn) for fn in DEVICES_DATA.filenames])
         lst.append('  - Total devices: {}'.format(len(DEVICES_DATA)))
         Printer.print(lst)
@@ -370,6 +384,7 @@ class UnrealDevice:
         self.data = None
         self.table = dict()
         self.testcase = ''
+        self.success_code = 0
 
     @property
     def is_connected(self):
@@ -392,6 +407,9 @@ class UnrealDevice:
 
         if self.address in DEVICES_DATA:
             self.data = DEVICES_DATA.get(self.address)
+            name = self.data.get('name', '')
+            name and setattr(self, 'name', name)
+
             self._is_connected = True
 
             testcase = kwargs.get('testcase', '')
@@ -399,20 +417,31 @@ class UnrealDevice:
                 if testcase in self.data.get('testcases', dict()):
                     self.testcase = testcase
                 else:
-                    fmt = '*** "{}" test case is unavailable for this connection ***'
-                    print(fmt.format(testcase))
+                    fmt = '"{}" test case is unavailable for this connection ***'
+                    Printer.print_message(fmt, testcase, prefix='UnrealDeviceWarning:')
 
             if kwargs.get('showed', True):
                 login_result = self.data.get('login', '')
-                if login_result:
-                    is_timestamp = kwargs.get('is_timestamp', True)
-                    login_result = self.render_data(
-                        login_result, is_timestamp=is_timestamp
-                    )
-                    print(login_result)
+                fmt = 'login unreal-device {}@dummy_username:dummy_password'
+                extra = fmt.format(self.address)
+                if testcase:
+                    if testcase in self.data.get('testcases', dict()):
+                        extra = '{}@testcase={}'.format(extra, testcase)
+                    else:
+                        extra = '{} fallback to default.'.format(extra)
+
+                is_timestamp = kwargs.get('is_timestamp', True)
+                login_result = self.render_data(
+                    login_result, is_timestamp=is_timestamp,
+                    service='authentication',
+                    extra=extra
+                )
+                print(login_result)
+            self.success_code = 0
             return self.is_connected
         else:
-            fmt = '{} is unavailable for connection.'
+            self.success_code = 1
+            fmt = '"{}" is unavailable for connection.'
             raise UnrealDeviceConnectionError(fmt.format(self.name))
 
     def reconnect(self, **kwargs):
@@ -428,30 +457,26 @@ class UnrealDevice:
         """
         if self.address in DEVICES_DATA:
             self.data = DEVICES_DATA.get(self.address)
-            self._is_connected = True
-
-            testcase = kwargs.get('testcase', '')
-            if testcase:
-                if testcase in self.data.get('testcases', dict()):
-                    self.testcase = testcase
-                else:
-                    fmt = '*** "{}" test case is unavailable for this reconnection ***'
-                    print(fmt.format(testcase))
 
             if kwargs.get('showed', True):
                 reload_txt = self.data.get('reload', '')
                 if not reload_txt:
                     reload_txt = kwargs.get('reload_data', '')
-                login_txt = self.data.get('login', '')
-                reconnect_txt = '{}\n{}'.format(reload_txt, login_txt).strip()
-                if reconnect_txt:
+
+                if reload_txt:
                     is_timestamp = kwargs.get('is_timestamp', True)
                     reconnect_txt = self.render_data(
-                        reconnect_txt, is_timestamp=is_timestamp
+                        reload_txt, is_timestamp=is_timestamp,
+                        service='reload', extra='reload unreal-device'
                     )
-                    print(reconnect_txt)
+                    print('{}\n\n'.format(reconnect_txt))
+
+            self._is_connected = False
+            self.connect(testcase=kwargs.get('testcase', ''))
+
             return self.is_connected
         else:
+            self.success_code = 1
             fmt = '{} is unavailable for reconnection.'
             raise UnrealDeviceConnectionError(fmt.format(self.name))
 
@@ -470,8 +495,13 @@ class UnrealDevice:
         if kwargs.get('showed', True):
             is_timestamp = kwargs.get('is_timestamp', True)
             msg = '{} is disconnected.'.format(self.name)
-            msg = self.render_data(msg, is_timestamp=is_timestamp)
+            msg = self.render_data(
+                msg, is_timestamp=is_timestamp,
+                service='authentication',
+                extra='logout unreal-device {}'.format(self.address),
+            )
             print(msg)
+        self.success_code = 0
         return self._is_connected
 
     @check_active_device
@@ -488,12 +518,27 @@ class UnrealDevice:
         str: output of a command line
         """
 
-        data = self.data.get('cmdlines')
+        if not cmdline.strip():
+            is_timestamp = kwargs.get('is_timestamp', True)
+            output = self.render_data(
+                cmdline, is_timestamp=is_timestamp,
+                service='execution', extra=cmdline,
+            )
+            if kwargs.get('showed', True):
+                print(output)
+            self.success_code = 0
+            return output
+
+        data = self.data.get('cmdlines', dict())
         if self.testcase:
             data = self.data.get('testcases').get(self.testcase, data)
 
-        no_output = '*** "{}" does not have output ***'.format(cmdline)
+        no_output = Printer.get_message('"{}" does not have output', cmdline,
+                                        prefix='UnrealDeviceCmdline:')
+
         result = data.get(cmdline, self.data.get('cmdlines').get(cmdline, no_output))
+        self.success_code = 1 if str(result).endswith('" does not have output') else 0
+
         if not isinstance(result, (list, tuple)):
             output = str(result)
         else:
@@ -503,9 +548,13 @@ class UnrealDevice:
             output = result[index]
 
         is_timestamp = kwargs.get('is_timestamp', True)
-        output = self.render_data(output, is_timestamp=is_timestamp)
+        output = self.render_data(
+            output, is_timestamp=is_timestamp,
+            service='execution', extra=cmdline,
+        )
         if kwargs.get('showed', True):
             print(output)
+
         return output
 
     @check_active_device
@@ -521,16 +570,40 @@ class UnrealDevice:
         -------
         str: result of configuration
         """
+
+        if Misc.is_list_instance(config):
+            config = '\n'.join(str(item) for item in config)
+        else:
+            config = str(config)
+
+        if kwargs.get('from_console_cmdline', False):
+            pattern = r'(\\r\\n)|\\r|\\n'
+            config = re.sub(pattern, '\n', config)
+
+        config = config.strip()
+
+        if not config:
+            config = 'configure\nend'
+
+        if not re.match(r'(?i)conf(i(g(u(r(e)?)?)?)?)?', config):
+            config = 'configure\n{}'.format(config)
+
+        if not config.splitlines()[-1].lower() == 'end':
+            config = '{}\nend'.format(config)
+
         is_timestamp = kwargs.get('is_timestamp', True)
-        result = self.render_data(config, is_cfg=True, is_timestamp=is_timestamp)
+        result = self.render_data(config, is_timestamp=is_timestamp, service='configuration')
         if kwargs.get('showed', True):
             print(result)
+
+        self.success_code = 0
         return result
 
-    def render_data(self, data, is_cfg=False, is_timestamp=True):
+    def render_data(self, data, extra=None, service='execution', is_timestamp=True):
 
         if isinstance(data, str):
             lst = data.splitlines()
+            lst = lst or ['']
         else:
             lst = []
             for item in data:
@@ -539,7 +612,7 @@ class UnrealDevice:
                 else:
                     lst.extend(item)
 
-        if is_cfg:
+        if service == 'configuration':
             prompt = '{}(configure)#'.format(self.name)
             
             for index, item in enumerate(lst):
@@ -549,10 +622,14 @@ class UnrealDevice:
 
         if is_timestamp:
             dt = datetime.now()
-            fmt = '+++ {:%b %d %Y %T}.{} from "unreal-device" for "{}"'
-            timestamp = fmt.format(dt, str(dt.microsecond)[:3], self.name)
-            lst.insert(int(is_cfg), timestamp)
-        
+            fmt = '{:%b %d %Y %T}.{} for "{}" - UNREAL-DEVICE-{}-SERVICE-TIMESTAMP'
+            timestamp = fmt.format(dt, str(dt.microsecond)[:3], self.name, service.upper())
+            index = 1 if service == 'configuration' else 0
+            lst.insert(index, timestamp)
+
+        if extra is not None:
+            lst.insert(0, extra)
+
         result = '\n'.join(lst)
         return result
 
@@ -638,3 +715,35 @@ def configure(device, config, **kwargs):
     """
     result = device.configure(config, **kwargs)
     return result
+
+
+def reconnect(device, **kwargs):
+    """Reconnect an unreal device
+
+    Parameters
+    ----------
+    device (UnrealDevice): an unreal device instance
+    kwargs (dict): keyword arguments
+
+    Returns
+    -------
+    bool: connection status
+    """
+    result = device.reconnect(**kwargs)
+    return result
+
+
+def reload(device, **kwargs):
+    """Reload an unreal device
+
+    Parameters
+    ----------
+    device (UnrealDevice): an unreal device instance
+    kwargs (dict): keyword arguments
+
+    Returns
+    -------
+    bool: connection status
+    """
+    result = device.reconnect(**kwargs)
+    return result
